@@ -122,6 +122,9 @@ row_spec_html <- function(kable_input, row, bold, italic, monospace,
 xml_cell_style <- function(x, bold, italic, monospace,
                            underline, strikeout, color, background,
                            align, font_size, angle, extra_css) {
+  if (is.na(xml_attr(x, "style"))) {
+    xml_attr(x, "style") <- ""
+  }
   if (bold) {
     xml_attr(x, "style") <- paste0(xml_attr(x, "style"),
                                    "font-weight: bold;")
@@ -187,14 +190,23 @@ row_spec_latex <- function(kable_input, row, bold, italic, monospace,
     table_info <- dup_fx_out[[2]]
   }
 
-  row <- row + 1
+  row <- row + table_info$position_offset
   for (i in row) {
     target_row <- table_info$contents[i]
-    new_row <- latex_new_row_builder(target_row, bold, italic, monospace,
+    new_row <- latex_new_row_builder(target_row, table_info,
+                                     bold, italic, monospace,
                                      underline, strikeout,
                                      color, background, align, font_size, angle,
                                      hline_after, extra_latex_after)
-    out <- sub(paste0(target_row, "\\\\\\\\"), new_row, out, perl = T)
+    temp_sub <- ifelse(i == 1 & table_info$tabular == "longtable", gsub, sub)
+    if (length(new_row) == 1) {
+      out <- temp_sub(target_row, new_row, out, perl = T)
+      table_info$contents[i] <- new_row
+    } else {
+      out <- temp_sub(paste0(target_row, "\\\\\\\\"),
+                  paste(new_row, collapse = ""), out, perl = T)
+      table_info$contents[i] <- new_row[1]
+    }
   }
 
   out <- structure(out, format = "latex", class = "knitr_kable")
@@ -202,56 +214,69 @@ row_spec_latex <- function(kable_input, row, bold, italic, monospace,
   return(out)
 }
 
-latex_new_row_builder <- function(target_row, bold, italic, monospace,
+latex_new_row_builder <- function(target_row, table_info,
+                                  bold, italic, monospace,
                                   underline, strikeout,
                                   color, background, align, font_size, angle,
                                   hline_after, extra_latex_after) {
   new_row <- latex_row_cells(target_row)
   if (bold) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\textbf{", x, "}")
+      paste0("\\\\textbf\\{", x, "\\}")
     })
   }
   if (italic) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\em{", x, "}")
+      paste0("\\\\em\\{", x, "\\}")
     })
   }
   if (monospace) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\ttfamily{", x, "}")
+      paste0("\\\\ttfamily\\{", x, "\\}")
     })
   }
   if (underline) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\underline{", x, "}")
+      paste0("\\\\underline\\{", x, "\\}")
     })
   }
   if (strikeout) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\sout{", x, "}")
+      paste0("\\\\sout\\{", x, "\\}")
     })
   }
   if (!is.null(color)) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\textcolor", latex_color(color), "{", x, "}")
+      paste0("\\\\textcolor", latex_color(color), "\\{", x, "\\}")
     })
   }
   if (!is.null(font_size)) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\begingroup\\\\fontsize{", font_size, "}{",
+      paste0("\\\\begingroup\\\\fontsize\\{", font_size, "\\}\\{",
              as.numeric(font_size) + 2,
-             "}\\\\selectfont ", x, "\\\\endgroup")})
+             "\\}\\\\selectfont ", x, "\\\\endgroup")})
   }
   if (!is.null(align)) {
+    if (!is.null(table_info$column_width)) {
+      p_align <- switch(align,
+                        "l" = "\\\\raggedright\\\\arraybackslash",
+                        "c" = "\\\\centering\\\\arraybackslash",
+                        "r" = "\\\\raggedleft\\\\arraybackslash")
+      align <- rep(align, table_info$ncol)
+      p_cols <- as.numeric(sub("column_", "", names(table_info$column_width)))
+      for (i in 1:length(p_cols)) {
+        align[p_cols[i]] <- paste0("\\>\\{", p_align, "\\}p\\{",
+                                   table_info$column_width[[i]], "\\}")
+      }
+    }
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\multicolumn{1}{", align, "}{", x, "}")
+      paste0("\\\\multicolumn\\{1\\}\\{", align, "\\}\\{", x, "\\}")
     })
   }
 
   if (!is.null(angle)) {
     new_row <- lapply(new_row, function(x) {
-      paste0("\\\\rotatebox{", angle, "}{", x, "}")
+      paste0("\\\\rotatebox\\{", angle, "\\}\\{", x, "\\}")
     })
   }
 
@@ -261,14 +286,18 @@ latex_new_row_builder <- function(target_row, bold, italic, monospace,
     new_row <- paste0("\\\\rowcolor", latex_color(background), "  ", new_row)
   }
 
-  new_row <- paste0(new_row, "\\\\\\\\")
-
-  if (hline_after) {
-    new_row <- paste0(new_row, "\n\\\\hline")
+  if (!hline_after & is.null(extra_latex_after)) {
+    return(new_row)
+  } else {
+    latex_after <- "\\\\\\\\"
+    if (hline_after) {
+      latex_after <- paste0(latex_after, "\n\\\\hline")
+    }
+    if (!is.null(extra_latex_after)) {
+      latex_after <- paste0(latex_after, "\n",
+                            regex_escape(extra_latex_after,
+                                         double_backslash = TRUE))
+    }
+    return(c(new_row, latex_after))
   }
-  if (!is.null(extra_latex_after)) {
-    new_row <- paste0(new_row, "\n",
-                      regex_escape(extra_latex_after, double_backslash = TRUE))
-  }
-  return(new_row)
 }
